/*
	File:		GraphicsCoreUtils.h

	Contains:	Declarations for 'utility' routines that portions of the GDX model (core, OSS, HAL)
				might want to make use of.

	Written by:	Sean Williams, Kevin Williams

	Copyright:	 1994-1995 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <2>	 7/17/95	SW		Added declaration for GraphicsUtil GetDefaultGammaTableID()
		 <1>	 4/15/95	SW		First Checked In

*/

#ifndef __GRAPHICSCOREUTILS__
#define __GRAPHICSCOREUTILS__
#include "GraphicsPriv.h"

#include <Types.h>
#include <Video.h>


// The following are prototypes for utility functions that the Core uses.

GDXErr GraphicsUtilCheckSetEntry(const VDSetEntryRecord *setEntry, UInt32 bitsPerPixel,
		SInt16 *startPosition, SInt16 *numberOfEntries, Boolean *sequential);

GDXErr GraphicsUtilSetEntries(const VDSetEntryRecord *setEntry, const GammaTbl *gamma, 
		DepthMode depthMode, UInt32 bitsPerPixel, Boolean luminanceMapping, Boolean directColor);
		
GDXErr GraphicsUtilBlackToWhiteRamp(const GammaTbl *gamma, DepthMode depthMode,
		UInt32 bitsPerPixel, Boolean luminanceMapping, Boolean directColor);
		
GDXErr GraphicsUtilGetDefaultGammaTableID(DisplayCode displayCode, GammaTableID *gammaTableID);

GDXErr GraphicsUtilMapSenseCodesToDisplayCode(RawSenseCode rawSenseCode,
		ExtendedSenseCode extendedSenseCode, DisplayCode *displayCode);

#endif	// __GRAPHICSCOREUTILS__
